from argparse import ArgumentParser
from collections import defaultdict
from io import StringIO
from pathlib import Path
from numpy import nan
import pandas as pd
import yaml

from parser import parse_mother_request


df = None

def _get_parser() -> ArgumentParser:
    parser = ArgumentParser()
    parser.add_argument(
        "path",
        help="Path for workflow. Serves as input and output.",
        type=str,
    )
    parser.add_argument(
        "--flat",
        help=(
            "Write application yaml as a single file, "
            "as opposed to in separate files (opa.yml, gsv.yml, meta.yml) "
            "in a same directory."
        )
    )
    return parser


def _row_to_dict(row: pd.Series, prefix: str = ""):
    # Pandas row to dict
    dct = row.to_dict()
    # Remove unused fields
    dct = {key: value for key, value in dct.items() if value is not nan}
    # Remove prefix from keys
    if prefix:
        dct = {key.removeprefix(prefix): value for key, value in dct.items()}
    return dct


def create_yaml_dicts(path: Path):
    global df  # pylint: disable=W0603

    mother_request = path / "conf" / "mother_request.yml"
    df = parse_mother_request(mother_request)
    opa_columns = [col for col in df.columns if col.startswith("opa_")]
    gsv_columns = [col for col in df.columns if col.startswith("gsv_")]
    dcts = defaultdict(dict)
    for app, app_df in df.groupby("Application"):
        for i, (_, request) in enumerate(app_df.iterrows(), start=1):
            dcts[app][i] = {
                    "GSVREQUEST": _row_to_dict(request[gsv_columns], "gsv_"),
                    "OPAREQUEST": _row_to_dict(request[opa_columns], "opa_"),
            }
    return dcts


def write_yamls_flat(path: Path):
    yamls = create_yaml_dicts(path)
    yamls_path = path / "conf" / "applications"
    for app, yaml_content in yamls.items():
        yaml_filename = f"{app.lower()}.yml"
        yaml_filepath = yamls_path / yaml_filename
        yaml_io = StringIO()
        yaml.safe_dump(yaml_content, yaml_io, default_flow_style=False)
        with open(yaml_filepath, "w", encoding="utf-8") as wf:
            wf.write(
                "---\n\n"
            )
            wf.write(
                yaml_io.getvalue().replace("\'", "\"")
            )


def write_yamls_dir(path: Path):
    yamls = create_yaml_dicts(path)
    conf_apps_path = path / "conf" / "applications"
    request_filename = [
        ("GSVREQUEST", "gsv.yml"),
        ("OPAREQUEST", "opa.yml"),
    ]
    for app, app_requests in yamls.items():
        app_path = conf_apps_path / app.lower()
        app_path.mkdir(exist_ok=True, parents=False)
        print(f"Made dir: {app_path}")
        for i, request in app_requests.items():
            request_path = app_path / f"{i:02d}"
            request_path.mkdir(exist_ok=True)
            for req, filename in request_filename:
                with open(request_path / filename, "w", encoding="utf-8") as wf:
                    print(request[req])
                    yaml.dump(request[req], wf)
        with open(app_path / "meta.yml", "w", encoding="utf-8") as wf:
            yaml.dump({
                "APPNAME": app,
            })


if __name__ == "__main__":
    args = _get_parser().parse_args()
    if args.flat:
        write_yamls_flat(Path(args.path))
    else:
        write_yamls_dir(Path(args.path))
